@@ -22,10 +22,11 @@
 #include <stdlib.h>
 #include <glib/gi18n-lib.h>
 #include "gnome-db-data-cell-renderer-info.h"
-#include "marshal.h"
+#include <libgnomedb/marshal.h>
 #include <libgda/libgda.h>
 #include <gdk/gdkkeysyms.h>
 #include "utility.h"
+#include <libgda/gda-enum-types.h>
 
 static void gnome_db_data_cell_renderer_info_get_property  (GObject *object,
 							    guint param_id,
@@ -72,7 +73,10 @@ enum {
 	PROP_ZERO,
 	PROP_VALUE_ATTRIBUTES,
 	PROP_EDITABLE,
-	PROP_TO_BE_DELETED
+	PROP_TO_BE_DELETED,
+  PROP_STORE,
+  PROP_ITER,
+  PROP_GROUP
 };
 
 struct _GnomeDbDataCellRendererInfoPriv {
@@ -148,8 +152,8 @@ gnome_db_data_cell_renderer_info_class_init (GnomeDbDataCellRendererInfoClass *c
   
 	g_object_class_install_property (object_class,
 					 PROP_VALUE_ATTRIBUTES,
-					 g_param_spec_uint ("value_attributes", NULL, NULL,
-                                                            0, G_MAXUINT, 0, G_PARAM_READWRITE));
+					 g_param_spec_flags ("value_attributes", NULL, NULL, GDA_TYPE_VALUE_ATTRIBUTE,
+                                                            GDA_VALUE_ATTR_NONE, G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_EDITABLE,
@@ -162,16 +166,30 @@ gnome_db_data_cell_renderer_info_class_init (GnomeDbDataCellRendererInfoClass *c
 					 g_param_spec_boolean ("to_be_deleted", NULL, NULL, FALSE,
                                                                G_PARAM_WRITABLE));
 
+	g_object_class_install_property (object_class,
+					 PROP_STORE,
+					 g_param_spec_object ("store", NULL, NULL, GNOME_DB_TYPE_DATA_STORE,
+                                                               G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_ITER,
+					 g_param_spec_object ("iter", NULL, NULL, GDA_TYPE_DATA_MODEL_ITER,
+                                                               G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY));
+
+	/* Ideally, GdaPropertyListGroup would be a boxed type, but it is not yet, so we use g_param_spec_pointer. */
+	g_object_class_install_property (object_class,
+					 PROP_GROUP,
+					 g_param_spec_pointer ("group", NULL, NULL,
+                                                               G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY));
 	info_cell_signals[STATUS_CHANGED] =
 		g_signal_new ("status_changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GnomeDbDataCellRendererInfoClass, status_changed),
 			      NULL, NULL,
-			      marshal_VOID__STRING_UINT,
+			      marshal_VOID__STRING_ENUM,
 			      G_TYPE_NONE, 2,
 			      G_TYPE_STRING,
-			      G_TYPE_UINT);
+			      GDA_TYPE_VALUE_ATTRIBUTE);
 }
 
 static void
@@ -187,8 +205,16 @@ gnome_db_data_cell_renderer_info_dispose (GObject *object)
 {
 	GnomeDbDataCellRendererInfo *cellinfo = GNOME_DB_DATA_CELL_RENDERER_INFO (object);
 
-	if (cellinfo->priv->iter)
+	if (cellinfo->priv->store) {
+		g_object_unref (cellinfo->priv->store);
+		cellinfo->priv->store = NULL;
+	}
+
+
+	if (cellinfo->priv->iter) {
 		iter_destroyed_cb (cellinfo->priv->iter, cellinfo);
+		cellinfo->priv->iter = NULL;
+	}
 
 	/* parent class */
 	parent_class->dispose (object);
@@ -219,7 +245,7 @@ gnome_db_data_cell_renderer_info_get_property (GObject *object,
   
 	switch (param_id) {
 	case PROP_VALUE_ATTRIBUTES:
-		g_value_set_uint (value, cellinfo->priv->attributes);
+		g_value_set_flags (value, cellinfo->priv->attributes);
 		break;
 	case PROP_EDITABLE:
 		g_value_set_boolean (value, cellinfo->priv->active);
@@ -241,7 +267,7 @@ gnome_db_data_cell_renderer_info_set_property (GObject *object,
   
 	switch (param_id) {
 	case PROP_VALUE_ATTRIBUTES:
-		cellinfo->priv->attributes = g_value_get_uint (value);
+		cellinfo->priv->attributes = g_value_get_flags (value);
 		g_object_set (object, "sensitive", 
 			      !(cellinfo->priv->attributes & GDA_VALUE_ATTR_NO_MODIF), NULL);
 		break;
@@ -251,6 +277,28 @@ gnome_db_data_cell_renderer_info_set_property (GObject *object,
 		break;
 	case PROP_TO_BE_DELETED:
 		break;
+	case PROP_STORE:
+		if (cellinfo->priv->store)
+			g_object_unref (cellinfo->priv->store);
+
+		cellinfo->priv->store = GNOME_DB_DATA_STORE(g_value_get_object(value));
+		if(cellinfo->priv->store)
+			g_object_ref(cellinfo->priv->store);
+    		break;
+	case PROP_ITER:
+		if (cellinfo->priv->iter)
+			g_object_unref(cellinfo->priv->iter);
+
+		cellinfo->priv->iter = GDA_DATA_MODEL_ITER(g_value_get_object(value));
+		if( cellinfo->priv->iter)
+			g_object_ref (cellinfo->priv->iter);
+
+		gda_object_connect_destroy (GDA_OBJECT (cellinfo->priv->iter), 
+				    G_CALLBACK (iter_destroyed_cb), cellinfo);
+    		break;
+	case PROP_GROUP:
+		cellinfo->priv->group = GDA_PARAMETER_LIST_GROUP(g_value_get_pointer(value));
+   		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -278,20 +326,13 @@ gnome_db_data_cell_renderer_info_new (GnomeDbDataStore *store,
 				      GdaDataModelIter *iter, GdaParameterListGroup *group)
 {
 	GObject *obj;
-	GnomeDbDataCellRendererInfo *info;
 
 	g_return_val_if_fail (GNOME_DB_IS_DATA_STORE (store), NULL);
 	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (iter), NULL);
 	g_return_val_if_fail (group && g_slist_find (((GdaParameterList *) iter)->groups_list, group), NULL);
 
-	obj = g_object_new (GNOME_DB_TYPE_DATA_CELL_RENDERER_INFO, NULL);
-	info = GNOME_DB_DATA_CELL_RENDERER_INFO (obj);
-	info->priv->store = store;
-
-	info->priv->iter = iter;
-	info->priv->group = group;
-	gda_object_connect_destroy (GDA_OBJECT (iter), 
-				    G_CALLBACK (iter_destroyed_cb), obj);
+	obj = g_object_new (GNOME_DB_TYPE_DATA_CELL_RENDERER_INFO, 
+                      "store", store, "iter", iter, "group", group, NULL);
 
 	return (GtkCellRenderer *) obj;
 }
@@ -350,7 +391,7 @@ gnome_db_data_cell_renderer_info_render (GtkCellRenderer      *cell,
 	GtkStyle *style;
 
 	if (!colors)
-		colors = utility_entry_build_info_colors_array ();
+		colors = gnome_db_utility_entry_build_info_colors_array ();
 
 	style = gtk_style_copy (widget->style);
 	orig_normal = & (style->bg[GTK_STATE_NORMAL]);
@@ -378,7 +419,7 @@ gnome_db_data_cell_renderer_info_render (GtkCellRenderer      *cell,
 	style->bg[GTK_STATE_NORMAL] = *normal;
 	style->bg[GTK_STATE_ACTIVE] = *normal;
 	style->bg[GTK_STATE_PRELIGHT] = *prelight;
-	gtk_style_attach (style, window);
+	style = gtk_style_attach (style, window); /* Note that we must use the return value, because this function is documented as sometimes returning a new object. */
 	gnome_db_data_cell_renderer_info_get_size (cell, widget, cell_area,
 						   &x_offset, &y_offset,
 						   &width, &height);
@@ -455,13 +496,13 @@ gnome_db_data_cell_renderer_info_activate (GtkCellRenderer      *cell,
 					    &attributes, -1);
 		}
 		else 
-			attributes = utility_proxy_compute_attributes_for_group (cellinfo->priv->group, 
+			attributes = gnome_db_utility_proxy_compute_attributes_for_group (cellinfo->priv->group, 
 										 cellinfo->priv->store,
 										 cellinfo->priv->iter,
 										 &iter, NULL);
 		
 		/* build the popup menu */
-		menu = utility_entry_build_actions_menu (G_OBJECT (cellinfo), attributes, 
+		menu = gnome_db_utility_entry_build_actions_menu (G_OBJECT (cellinfo), attributes, 
 							 G_CALLBACK (mitem_activated_cb));
 		g_object_set_data (G_OBJECT (cellinfo), "path", g_strdup (path));
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
