@@ -60,7 +60,7 @@ static void gnome_db_entry_string_start_editing (GtkCellEditable *iface, GdkEven
 static GtkWidget *create_entry (GnomeDbEntryWrapper *mgwrap);
 static void       real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value);
 static GValue    *real_get_value (GnomeDbEntryWrapper *mgwrap);
-static void       connect_signals(GnomeDbEntryWrapper *mgwrap, GCallback modif_cb, GCallback activ_cb);
+static void       connect_signals(GnomeDbEntryWrapper *mgwrap, GCallback modify_cb, GCallback activate_cb);
 static gboolean   expand_in_layout (GnomeDbEntryWrapper *mgwrap);
 static void       set_editable (GnomeDbEntryWrapper *mgwrap, gboolean editable);
 static void       grab_focus (GnomeDbEntryWrapper *mgwrap);
@@ -440,34 +440,34 @@ typedef void (*Callback2) (gpointer, gpointer);
 static gboolean
 focus_out_cb (GtkWidget *widget, GdkEventFocus *event, GnomeDbEntryString *mgstr)
 {
-	GCallback activ_cb;
-	activ_cb = g_object_get_data (G_OBJECT (widget), "_activ_cb");
-	g_assert (activ_cb);
-	((Callback2)activ_cb) (widget, mgstr);
+	GCallback activate_cb;
+	activate_cb = g_object_get_data (G_OBJECT (widget), "_activate_cb");
+	g_assert (activate_cb);
+	((Callback2)activate_cb) (widget, mgstr);
 
 	return FALSE;
 }
 
 static void
-connect_signals(GnomeDbEntryWrapper *mgwrap, GCallback modif_cb, GCallback activ_cb)
+connect_signals(GnomeDbEntryWrapper *mgwrap, GCallback modify_cb, GCallback activate_cb)
 {
 	GnomeDbEntryString *mgstr;
 
 	g_return_if_fail (GNOME_DB_IS_ENTRY_STRING (mgwrap));
 	mgstr = GNOME_DB_ENTRY_STRING (mgwrap);
 	g_return_if_fail (mgstr->priv);
-	g_object_set_data (G_OBJECT (mgstr->priv->entry), "_activ_cb", activ_cb);
-	g_object_set_data (G_OBJECT (mgstr->priv->view), "_activ_cb", activ_cb);
+	g_object_set_data (G_OBJECT (mgstr->priv->entry), "_activate_cb", activate_cb);
+	g_object_set_data (G_OBJECT (mgstr->priv->view), "_activate_cb", activate_cb);
 
 	g_signal_connect (G_OBJECT (mgstr->priv->entry), "changed",
-			  modif_cb, mgwrap);
+			  modify_cb, mgwrap);
 	g_signal_connect (G_OBJECT (mgstr->priv->entry), "activate",
-			  activ_cb, mgwrap);
+			  activate_cb, mgwrap);
 	g_signal_connect (G_OBJECT (mgstr->priv->entry), "focus-out-event",
 			  G_CALLBACK (focus_out_cb), mgstr);
 
 	g_signal_connect (G_OBJECT (mgstr->priv->buffer), "changed",
-			  modif_cb, mgwrap);
+			  modify_cb, mgwrap);
 	g_signal_connect (G_OBJECT (mgstr->priv->view), "focus-out-event",
 			  G_CALLBACK (focus_out_cb), mgstr);
 	/* FIXME: how does the user "activates" the GtkTextView widget ? */
