@@ -53,6 +53,7 @@ enum
         PROP_0,
         PROP_RAW_FORM,
 	PROP_INFO,
+	PROP_MODEL
 };
 
 GType
@@ -87,22 +88,40 @@ gnome_db_form_class_init (GnomeDbFormClass *class)
 	parent_class = g_type_class_peek_parent (class);
 
 	/* Properties */
+	
         object_class->set_property = gnome_db_form_set_property;
         object_class->get_property = gnome_db_form_get_property;
 	g_object_class_install_property (object_class, PROP_RAW_FORM,
-                                         g_param_spec_pointer ("raw_form", NULL, NULL,
+                                         g_param_spec_object ("raw_form", NULL, NULL, GNOME_DB_TYPE_RAW_FORM,
                                                                G_PARAM_READABLE));
 	g_object_class_install_property (object_class, PROP_INFO,
-                                         g_param_spec_pointer ("widget_info", NULL, NULL,
-							       G_PARAM_READABLE));
+                                         g_param_spec_object ("widget_info", NULL, NULL, GNOME_DB_TYPE_DATA_WIDGET,
+							                                   G_PARAM_READABLE));
+
+											g_object_class_install_property (object_class, PROP_MODEL,
+											                                 g_param_spec_object ("model", NULL, NULL,
+											                                                      GDA_TYPE_DATA_MODEL,
+											                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
-gnome_db_form_init (GnomeDbForm *wid)
+gnome_db_form_init (GnomeDbForm *form)
 {
-	wid->priv = g_new0 (GnomeDbFormPriv, 1);
-	wid->priv->raw_form = NULL;
-	wid->priv->info = NULL;
+	form->priv = g_new0 (GnomeDbFormPriv, 1);
+	form->priv->raw_form = NULL;
+	form->priv->info = NULL;
+	
+	form->priv->raw_form = gnome_db_raw_form_new (NULL);
+	gtk_box_pack_start (GTK_BOX (form), form->priv->raw_form, TRUE, TRUE, 0);
+	gtk_widget_show (form->priv->raw_form);
+
+	form->priv->info = gnome_db_data_widget_info_new (GNOME_DB_DATA_WIDGET (form->priv->raw_form), 
+							  GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW |
+							  GNOME_DB_DATA_WIDGET_INFO_ROW_MODIFY_BUTTONS |
+							  GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS);
+	gtk_box_pack_start (GTK_BOX (form), form->priv->info, FALSE, TRUE, 0);
+	gtk_widget_show (form->priv->info);
+
 }
 
 /**
@@ -117,22 +136,12 @@ GtkWidget *
 gnome_db_form_new (GdaDataModel *model)
 {
 	GnomeDbForm *form;
-
+	
 	g_return_val_if_fail (!model || GDA_IS_DATA_MODEL (model), NULL);
-
-	form = (GnomeDbForm *) g_object_new (GNOME_DB_TYPE_FORM, NULL);
-
-	form->priv->raw_form = gnome_db_raw_form_new (model);
-	gtk_box_pack_start (GTK_BOX (form), form->priv->raw_form, TRUE, TRUE, 0);
-	gtk_widget_show (form->priv->raw_form);
-
-	form->priv->info = gnome_db_data_widget_info_new (GNOME_DB_DATA_WIDGET (form->priv->raw_form), 
-							  GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW |
-							  GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS |
-							  GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS);
-	gtk_box_pack_start (GTK_BOX (form), form->priv->info, FALSE, TRUE, 0);
-	gtk_widget_show (form->priv->info);
-
+	
+	form = (GnomeDbForm *) g_object_new (GNOME_DB_TYPE_FORM,
+	                                     "model", model, NULL);
+	
 	return (GtkWidget *) form;
 }
 
@@ -144,15 +153,23 @@ gnome_db_form_set_property (GObject *object,
 				 GParamSpec *pspec)
 {
 	GnomeDbForm *form;
+	GdaDataModel *model;
 
-        form = GNOME_DB_FORM (object);
-        if (form->priv) {
-                switch (param_id) {
+	form = GNOME_DB_FORM (object);
+	
+	if (form->priv) {
+		switch (param_id) {
+		case PROP_MODEL:
+			model = GDA_DATA_MODEL (g_value_get_object (value));
+			g_object_set (G_OBJECT(form->priv->raw_form),
+			              "model", model, NULL);
+			break;
+			
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
-                }
-        }
+		}
+	}
 }
 
 static void
@@ -162,19 +179,25 @@ gnome_db_form_get_property (GObject *object,
 				 GParamSpec *pspec)
 {
 	GnomeDbForm *form;
+	GdaDataModel *model;
 
         form = GNOME_DB_FORM (object);
         if (form->priv) {
-                switch (param_id) {
-                case PROP_RAW_FORM:
-			g_value_set_pointer (value, form->priv->raw_form);
-                        break;
-                case PROP_INFO:
-			g_value_set_pointer (value, form->priv->info);
-                        break;
+		switch (param_id) {
+		case PROP_RAW_FORM:
+			g_value_set_object (value, form->priv->raw_form);
+			break;
+		case PROP_INFO:
+			g_value_set_object (value, form->priv->info);
+			break;
+		case PROP_MODEL:
+			g_object_get (G_OBJECT (form->priv->raw_form),
+			              "model", &model, NULL);
+			g_value_set_object (value, G_OBJECT (model));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
-                }
-        }	
+		}
+        }
 }
