@@ -1,7 +1,18 @@
 #include <glib/gi18n-lib.h>
 #include <libgnomedb/libgnomedb.h>
+#include <libgnomedb/gnome-db-plugin.h>
 #include <gtk/gtk.h>
 
+typedef struct {
+	GType   *tested_gtypes;
+	gint     nb_tested_gtypes;
+	GdaDict *dict;
+} TestConfig;
+
+
+extern GHashTable *gnome_db_plugins_hash;
+TestConfig mainconf;
+
 static gboolean delete_event( GtkWidget *widget,
                               GdkEvent  *event,
                               gpointer   data )
@@ -18,7 +29,8 @@ static void destroy( GtkWidget *widget,
 }
 
 GtkWidget *build_menu (GtkWidget *mainwin, GtkWidget *top_nb);
-GtkWidget *build_main (GdaDict *dict, GdaDataHandler *dh, GType type);
+GtkWidget *build_main_builtin (GdaDataHandler *dh, GType type, const gchar *plugin_name);
+static GtkWidget *build_main_plugin (GnomeDbPlugin *plugin);
 
 static guint
 gtype_hash (gconstpointer key)
@@ -64,24 +76,32 @@ get_handler (GType for_type)
 	return g_hash_table_lookup (hash, (gpointer) for_type);
 }
 
+static void plugin_hash_foreach_func (const gchar *plugin_name, GnomeDbPlugin *plugin, GtkWidget *nb);
+
 int 
 main (int argc, char **argv)
 {
 	GtkWidget *mainwin, *vbox, *wid, *nb, *label, *menu, *top_nb;
 	GdaDict *dict;
-	GSList *plugins, *list;
 	GType type;
 	GdaDataHandler *dh;
 	gint i;
-
+	
 	/* Initialize i18n support */
 	gtk_set_locale ();
 	
 	/* Initialize the widget set */
 	gnome_db_init ("test-handlers", "", argc, argv);
 
-	/* tested objects */
-	dict = GDA_DICT (gda_dict_new ());
+	/* init main conf */
+	dict = gda_dict_new ();
+	GType tested_gtypes [] = {G_TYPE_INT64, G_TYPE_UINT64, GDA_TYPE_BINARY, G_TYPE_BOOLEAN, GDA_TYPE_BLOB, G_TYPE_DATE, G_TYPE_DOUBLE,
+				  GDA_TYPE_GEOMETRIC_POINT, G_TYPE_OBJECT, G_TYPE_INT, GDA_TYPE_LIST, 
+				  GDA_TYPE_NUMERIC, G_TYPE_FLOAT, GDA_TYPE_SHORT, GDA_TYPE_USHORT, G_TYPE_STRING, 
+				  GDA_TYPE_TIME, GDA_TYPE_TIMESTAMP, G_TYPE_CHAR, G_TYPE_UCHAR, G_TYPE_UINT};
+	mainconf.tested_gtypes = tested_gtypes;
+	mainconf.nb_tested_gtypes = sizeof (tested_gtypes) / sizeof (GType);
+	mainconf.dict = dict;
 
 	/* Create the main window */
 	mainwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -126,17 +146,12 @@ main (int argc, char **argv)
 	gtk_widget_show (nb);
 
 	/* populating the notebook */
-	GType tested_gtypes [] = {G_TYPE_INT64, G_TYPE_UINT64, GDA_TYPE_BINARY, G_TYPE_BOOLEAN, GDA_TYPE_BLOB, G_TYPE_DATE, G_TYPE_DOUBLE,
-				  GDA_TYPE_GEOMETRIC_POINT, G_TYPE_OBJECT, G_TYPE_INT, GDA_TYPE_LIST, 
-				  GDA_TYPE_NUMERIC, G_TYPE_FLOAT, GDA_TYPE_SHORT, GDA_TYPE_USHORT, G_TYPE_STRING, 
-				  GDA_TYPE_TIME, GDA_TYPE_TIMESTAMP, G_TYPE_CHAR, G_TYPE_UCHAR, G_TYPE_UINT};
-
-	for (i = 0; i < sizeof (tested_gtypes) / sizeof (GType); i++) {
+	for (i = 0; i < mainconf.nb_tested_gtypes; i++) {
 		type = tested_gtypes [i];
 		dh = get_handler (type);
-		wid = build_main (dict, dh, type);
+		wid = build_main_builtin (dh, type, NULL);
 		gtk_widget_show (wid);
-		label = gtk_label_new (gda_type_to_string (type));
+		label = gtk_label_new (gda_g_type_to_string (type));
 		gtk_notebook_append_page (GTK_NOTEBOOK (nb), wid, label);
 	}
 
@@ -162,32 +177,7 @@ main (int argc, char **argv)
 	gtk_widget_show (nb);
 
 	/* populating the notebook */
-	/* plugins = gda_connection_MIG_get_plugin_handlers (srv); */
-/* 	list = plugins; */
-/* 	while (list) { */
-/* 		gint i, nbt; */
-/* 		gchar *str; */
-
-/* 		dh = GDA_DATA_HANDLER (list->data); */
-/* 		nbt = gda_data_handler_get_nb_gda_types (dh); */
-/* 		for (i=0; i<nbt; i++) { */
-/* 			type = gda_data_handler_get_gda_type_index (dh, i); */
-/* 			wid = build_main (dict, dh, type); */
-/* 			gtk_widget_show (wid); */
-/* 			str = g_strdup_printf ("%s\n(%s)", gda_data_handler_get_plugin_name (dh), */
-/* 					       gda_type_to_string (type)); */
-/* 			label = gtk_label_new (str); */
-/* 			gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5); */
-/* 			g_free (str); */
-/* 			gtk_notebook_append_page (GTK_NOTEBOOK (nb), wid, label); */
-/* 		} */
-/* 		list = g_slist_next (list); */
-/* 	} */
-/* 	g_slist_free (plugins); */
-		
-
-
-
+	g_hash_table_foreach (gnome_db_plugins_hash, (GHFunc) plugin_hash_foreach_func, nb);
 
 	/* Show the application window */
 	gtk_widget_show (mainwin);
@@ -202,20 +192,22 @@ void entry_contents_modified (GtkWidget *entry, gpointer data);
 void null_toggled_cb (GtkToggleButton *button, GtkWidget *entry);
 void default_toggled_cb (GtkToggleButton *button, GtkWidget *entry);
 void actions_toggled_cb (GtkToggleButton *button, GtkWidget *entry);
+void editable_toggled_cb (GtkToggleButton *button, GtkWidget *entry);
 void orig_clicked_cb (GtkButton *button, GtkWidget *entry);
 void default_clicked_cb (GtkButton *button, GtkWidget *entry);
 GtkWidget *
-build_main (GdaDict *dict, GdaDataHandler *dh, GType type)
+build_main_builtin (GdaDataHandler *dh, GType type, const gchar *plugin_name)
 {
 	GtkWidget *wid, *label, *table, *button, *bbox, *entry, *vbox;
 	GString *string;
+	gboolean expand;
 	
 	vbox = gtk_vbox_new (FALSE, 0);
 
-	table = gtk_table_new (7, 2, FALSE);
+	table = gtk_table_new (7, 3, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 5);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 5);
-	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
 
 	/* explain top label */
 	string = g_string_new ("");
@@ -233,11 +225,14 @@ build_main (GdaDict *dict, GdaDataHandler *dh, GType type)
 	gtk_widget_show (label);
 
 	/* widget being tested */
-	wid = GTK_WIDGET (gnome_db_util_new_data_entry (NULL, type, NULL));
-	if (gnome_db_data_entry_expand_in_layout (GNOME_DB_DATA_ENTRY (wid)))
-		gtk_table_attach_defaults (GTK_TABLE (table), wid, 0, 2, 1, 2);
+	wid = GTK_WIDGET (gnome_db_util_new_data_entry (mainconf.dict, type, plugin_name));
+	g_object_set (G_OBJECT (wid), "handler", dh, NULL);
+
+	expand = gnome_db_data_entry_expand_in_layout (GNOME_DB_DATA_ENTRY (wid));
+	if (expand)
+		gtk_table_attach_defaults (GTK_TABLE (table), wid, 0, 3, 1, 2);
 	else
-		gtk_table_attach (GTK_TABLE (table), wid, 0, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+		gtk_table_attach (GTK_TABLE (table), wid, 0, 3, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
 	gtk_widget_show (wid);
 
 	/* Other widgets */
@@ -248,18 +243,18 @@ build_main (GdaDict *dict, GdaDataHandler *dh, GType type)
 
 	label = gtk_label_new (_("--"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 2, 3);
+	gtk_table_attach (GTK_TABLE (table), label, 1, 3, 2, 3, 0, 0, 0, 0);
 	g_object_set_data (G_OBJECT (wid), "flags", label);
 	gtk_widget_show (label);
 
 	label = gtk_label_new (_("Current value: "));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, 0, 0, 0);
 	gtk_widget_show (label);
 	
 	label = gtk_label_new ("");
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 3, 4);
+	gtk_table_attach (GTK_TABLE (table), label, 1, 3, 3, 4, 0, 0, 0, 0);
 	g_object_set_data (G_OBJECT (wid), "value", label);
 	g_signal_connect (G_OBJECT (wid), "contents_modified",
 			  G_CALLBACK (entry_contents_modified), NULL);
@@ -268,7 +263,7 @@ build_main (GdaDict *dict, GdaDataHandler *dh, GType type)
 	entry_contents_modified (wid, GTK_LABEL (label));
 
 	bbox = gtk_hbutton_box_new ();
-	gtk_table_attach (GTK_TABLE (table), bbox, 0, 2, 4, 5, 0, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), bbox, 0, 3, 4, 5, 0, 0, 0, 0);
 		
 	button = gtk_toggle_button_new_with_label (_("NULL ok"));
 	g_signal_connect (G_OBJECT (button), "toggled",
@@ -293,35 +288,48 @@ build_main (GdaDict *dict, GdaDataHandler *dh, GType type)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				      gnome_db_data_entry_get_attributes (GNOME_DB_DATA_ENTRY (wid)) & 
 				      GDA_VALUE_ATTR_ACTIONS_SHOWN);
+
+	button = gtk_toggle_button_new_with_label (_("Editable?"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			  G_CALLBACK (editable_toggled_cb), wid);
+	gtk_container_add (GTK_CONTAINER (bbox), button);
+			      
 	gtk_widget_show_all (bbox);
 
 	/* to set the original value */
-	entry = GTK_WIDGET (gnome_db_util_new_data_entry (NULL, type, NULL));
-	gtk_table_attach (GTK_TABLE (table), entry, 0, 1, 5, 6, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+	entry = GTK_WIDGET (gnome_db_util_new_data_entry (mainconf.dict, type, plugin_name));
+	g_object_set (G_OBJECT (entry), "handler", dh, NULL);
+
+	gtk_table_attach (GTK_TABLE (table), entry, 0, 2, 5, 6, GTK_FILL | GTK_EXPAND, 0, 0, 0);
 	gtk_widget_show (entry);
 	button = gtk_button_new_with_label (_("Set as original"));
 	g_object_set_data (G_OBJECT (button), "entry", entry);
-	gtk_table_attach (GTK_TABLE (table), button, 1, 2, 5, 6, 0, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), button, 2, 3, 5, 6, 0, 0, 0, 0);
 	gtk_widget_show (button);
 	g_signal_connect (G_OBJECT (button), "clicked",
 			  G_CALLBACK (orig_clicked_cb), wid);
 
 	/* to set the default value */
-	entry = GTK_WIDGET (gnome_db_util_new_data_entry (NULL, type, NULL));
-	gtk_table_attach (GTK_TABLE (table), entry, 0, 1, 6, 7, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+	entry = GTK_WIDGET (gnome_db_util_new_data_entry (mainconf.dict, type, plugin_name));
+	g_object_set (G_OBJECT (entry), "handler", dh, NULL);
+
+	gtk_table_attach (GTK_TABLE (table), entry, 0, 2, 6, 7, GTK_FILL | GTK_EXPAND, 0, 0, 0);
 	gtk_widget_show (entry);
 	button = gtk_button_new_with_label (_("Set as default"));
 	g_object_set_data (G_OBJECT (button), "entry", entry);
-	gtk_table_attach (GTK_TABLE (table), button, 1, 2, 6, 7, 0, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), button, 2, 3, 6, 7, 0, 0, 0, 0);
 	gtk_widget_show (button);
 	g_signal_connect (G_OBJECT (button), "clicked",
 			  G_CALLBACK (default_clicked_cb), wid);
 
 	gtk_widget_show (table);
 
-	label = gtk_label_new("");
-	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
-	gtk_widget_show (label);
+	if (!expand) {
+		label = gtk_label_new ("");
+		gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+		gtk_widget_show (label);
+	}
 
 	gtk_widget_show (vbox);
 
@@ -370,6 +378,15 @@ actions_toggled_cb (GtkToggleButton *button, GtkWidget *entry)
 				      GDA_VALUE_ATTR_ACTIONS_SHOWN);
 }
 
+void
+editable_toggled_cb (GtkToggleButton *button, GtkWidget *entry)
+{
+	gboolean editable;
+
+	editable = gtk_toggle_button_get_active (button);
+	gnome_db_data_entry_set_editable (GNOME_DB_DATA_ENTRY (entry), editable);
+}
+
 void 
 entry_contents_modified (GtkWidget *entry, gpointer data)
 {
@@ -396,11 +413,14 @@ entry_contents_modified (GtkWidget *entry, gpointer data)
 
 	/* value */
 	label = g_object_get_data (G_OBJECT (entry), "value");
-	value = gnome_db_data_entry_get_value (GNOME_DB_DATA_ENTRY (entry));
 	dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (entry));
 	if (dh) {
 		gchar *strval;
-		strval = gda_data_handler_get_sql_from_value (dh, value);
+		value = gnome_db_data_entry_get_value (GNOME_DB_DATA_ENTRY (entry));
+		if (G_VALUE_TYPE (value) == GDA_TYPE_BINARY)
+			strval = g_strdup ("Binary data...");
+		else
+			strval = gda_data_handler_get_sql_from_value (dh, value);
 		gda_value_free (value);
 		gtk_label_set_text (label, strval);
 		g_free (strval);
@@ -417,14 +437,18 @@ orig_clicked_cb (GtkButton *button, GtkWidget *entry)
 	value = gnome_db_data_entry_get_value (GNOME_DB_DATA_ENTRY (wid));
 	g_print ("Setting to original Value %p ", value);
 	if (value) {
+		gchar *str = gda_value_stringify (value);
 		if (G_VALUE_TYPE (value) == GDA_TYPE_NULL)
-			g_print ("GDA_TYPE_NULL\n");
+			g_print ("GDA_TYPE_NULL");
 		else
-			g_print ("type = %d\n", G_VALUE_TYPE (value));
+			g_print ("%s", g_type_name (G_VALUE_TYPE (value)));
+		g_print (": %s\n", str);
+		g_free (str);
 	}
 	
 	gnome_db_data_entry_set_value_orig (GNOME_DB_DATA_ENTRY (entry), value);
 	gda_value_free (value);
+	entry_contents_modified (entry, NULL);
 }
 
 void
@@ -507,3 +531,199 @@ on_plugins1_activate (GtkMenuItem *item, GtkWidget *top_nb)
 {
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (top_nb), 1);
 }
+
+
+static void
+plugin_hash_foreach_func (const gchar *plugin_name, GnomeDbPlugin *plugin, GtkWidget *nb)
+{
+	GtkWidget *label, *wid;
+
+	if (plugin->plugin_file) {
+		wid = build_main_plugin (plugin);
+		gtk_widget_show (wid);
+		label = gtk_label_new (plugin_name);
+		gtk_notebook_append_page (GTK_NOTEBOOK (nb), wid, label);
+	}
+}
+
+
+static void create_plugin_nb (GtkWidget *vbox, GnomeDbPlugin *plugin);
+static void options_form_param_changed_cb (GnomeDbBasicForm *form, GdaParameter *param, gboolean is_user_modif, GtkWidget *vbox);
+static GtkWidget *
+build_main_plugin (GnomeDbPlugin *plugin)
+{
+	GtkWidget *label, *vbox, *hbox;
+	GString *string;
+
+	g_assert (plugin);
+	vbox = gtk_vbox_new (FALSE, 0);
+
+	/* explain top label */
+	string = g_string_new ("");
+	g_string_append_printf (string, "<b>Plugin Name:</b> %s",
+				plugin->plugin_name);
+	label = gtk_label_new ("");
+
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_label_set_markup (GTK_LABEL (label), string->str);
+	g_string_free (string, TRUE);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+	gtk_widget_show (label);
+
+	string = g_string_new ("");
+	g_string_append_printf (string, "<b>Plugin Description:</b> %s",
+				plugin->plugin_descr);
+	label = gtk_label_new ("");
+
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_label_set_markup (GTK_LABEL (label), string->str);
+	g_string_free (string, TRUE);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+	gtk_widget_show (label);
+
+	/* options */
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show (hbox);
+
+	string = g_string_new ("");
+	if (plugin->options_xml_spec) 
+		g_string_append_printf (string, "<b>Plugin Options:</b>");
+	else 
+		g_string_append_printf (string, "<b>Plugin Options:</b> None");
+
+	label = gtk_label_new ("");
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+	gtk_label_set_markup (GTK_LABEL (label), string->str);
+	g_string_free (string, TRUE);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_widget_show (label);
+
+	if (plugin->options_xml_spec) {
+		GdaParameterList *plist;
+		GError *error = NULL;
+
+		plist = gda_parameter_list_new_from_spec_string (mainconf.dict, plugin->options_xml_spec, &error);
+		if (!plist) {
+			g_warning ("Cannot parse XML spec for plugin options: %s",
+				   error && error->message ? error->message : "No detail");
+			g_error_free (error);
+		}
+		else {
+			GtkWidget *form;
+
+			form = gnome_db_basic_form_new (plist);
+			g_object_unref (plist);
+			gtk_box_pack_start (GTK_BOX (hbox), form, TRUE, TRUE, 0);
+			gtk_widget_show (form);
+			g_object_set_data (G_OBJECT (vbox), "options", form);
+			g_signal_connect (G_OBJECT (form), "param_changed",
+					  G_CALLBACK (options_form_param_changed_cb), vbox);
+		}
+	}
+
+	/* notebook */
+	create_plugin_nb (vbox, plugin);
+
+	gtk_widget_show (vbox);
+
+	return vbox;
+}
+
+static void
+create_plugin_nb (GtkWidget *vbox, GnomeDbPlugin *plugin)
+{
+	GtkWidget *wid, *nb, *label;
+	gint i;
+	GType type;
+	GdaDataHandler *dh;
+	GnomeDbBasicForm *options_form;
+	gchar *options = NULL;
+	gchar *plugin_name;
+	
+	options_form = g_object_get_data (G_OBJECT (vbox), "options");
+	if (options_form) {
+		GdaParameterList *plist;
+		GSList *list;
+		GString *string = NULL;
+
+		plist = gnome_db_basic_form_get_data_set (options_form);
+		g_assert (plist);
+		for (list = plist->parameters; list; list = list->next) {
+			GdaParameter *param = GDA_PARAMETER (list->data);
+			const GValue *value;
+			value = gda_parameter_get_value (param);
+			if (value && !gda_value_is_null (value)) {
+				gchar *str;
+				if (!string)
+					string = g_string_new ("");
+				else
+					g_string_append_c (string, ';');
+				g_string_append (string, gda_object_get_id (GDA_OBJECT (param)));
+				g_string_append_c (string, '=');
+				str = gda_value_stringify (value);
+				g_string_append (string, str);
+				g_free (str);
+			}
+		}
+
+		if (string) {
+			options = string->str;
+			g_string_free (string, FALSE);
+		}
+	}
+
+	if (options) {
+		plugin_name = g_strdup_printf ("%s:%s", plugin->plugin_name, options);
+		g_free (options);
+	}
+	else
+		plugin_name = g_strdup (plugin->plugin_name);
+
+	nb = gtk_notebook_new ();
+	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (nb), GTK_POS_LEFT);
+	gtk_box_pack_start (GTK_BOX (vbox), nb, TRUE, TRUE, 0);
+	gtk_widget_show (nb);
+
+	g_object_set_data (G_OBJECT (vbox), "nb", nb);
+	g_object_set_data (G_OBJECT (vbox), "plugin", plugin);
+
+	if (plugin->nb_g_types > 0)
+		for (i = 0; i < plugin->nb_g_types; i++) {
+			type = plugin->valid_g_types[i];
+			dh = get_handler (type);
+			if (dh) {
+				wid = build_main_builtin (dh, type, plugin_name);
+				gtk_widget_show (wid);
+				label = gtk_label_new (gda_g_type_to_string (type));
+				gtk_notebook_append_page (GTK_NOTEBOOK (nb), wid, label);
+			}
+		}
+	else
+		for (i = 0; i < mainconf.nb_tested_gtypes; i++) {
+			type = mainconf.tested_gtypes [i];
+			dh = get_handler (type);
+			if (dh) {
+				wid = build_main_builtin (dh, type, plugin_name);
+				gtk_widget_show (wid);
+				label = gtk_label_new (gda_g_type_to_string (type));
+				gtk_notebook_append_page (GTK_NOTEBOOK (nb), wid, label);
+			}
+		}
+	g_free (plugin_name);
+}
+
+static void
+options_form_param_changed_cb (GnomeDbBasicForm *form, GdaParameter *param, gboolean is_user_modif, GtkWidget *vbox)
+{
+	GtkWidget *nb;
+	GnomeDbPlugin *plugin;
+	
+	nb = g_object_get_data (G_OBJECT (vbox), "nb");
+	plugin = g_object_get_data (G_OBJECT (vbox), "plugin");
+	g_assert (plugin);
+	
+	if (nb)
+		gtk_widget_destroy (nb);
+	create_plugin_nb (vbox, plugin);
+}
